---------------------------------------------------------------------------------------------
QHVERSION = "QuickHeal v2.2.3.0"
---------------------------------------------------------------------------------------------
-- Author
--
--	WakeZero (Maleena on Kel'Thuzad | Force [Alliance])
-- 
-- Description
--
--	This mod allows a healer to quickly cast spells with a single mouse click.
--
-- Example
--
--	When enabled, damage taken bars will appear beside each player and party
--	unit frame. These bars increase with the amount of damage a player in the
--	party recieves, changing colors from green to yellow to red upon death.
--
--	By opening the options menu (/QH or assigned keybinding) the user may
--	choose spells from their spellbook to cast upon the corresponding party
--	member when the bars are clicked. Currently the Left, Middle and Right
--	mouse buttons may be assigned to.
--
-- TODO
--
--	* Add Shift-Alt-Control bindings
--	* Automatically bind left click to ressurection spell upon death
--	* Localize for other languages
--	* Add class specific bindings and options
--	* Add raid frames
--
-- Changelog
--
--	v2.2.3.0 [2007-10-25] : First Release
--
-- Shoutouts
--
--	Jesus Christ - For joy, life and eternal love (and the whole forgiveness of sins bit)
--
--	Curse (wow.curse.com) - For hosting my mod and many others.
--
--	WoWWiki (www.wowwiki.com) - Greatest resource and API for WoW mod developers.
--
--	Force (www.forceguild.net) - Grats on Illidan, keep kicking ass!
--
---------------------------------------------------------------------------------------------
-- Variables
---------------------------------------------------------------------------------------------
-- Player Profile
QHProfile = {};
PlayerName = nil;

-- Key Bindings
BINDING_HEADER_QuickHeal = "QuickHeal";

-- QuickHeal Options Variables
QHrunning = nil;
QHplayerleft = nil;
QHplayermiddle = nil;
QHplayerright = nil;
QHpartyleft = nil;
QHpartymiddle = nil;
QHpartyright = nil;

-- TargetFrame Position Variables
QHtfpoint = nil;
QHtfrelativeTo = nil;
QHtfrelativePoint = nil;
QHtfxofs = nil;
QHtfyofs = nil;

-- State Headers
stateheaders = {};
stateheaders[1] = CreateFrame("Frame", "StateHeader", nil, "SecureStateHeaderTemplate");
for i=2,5 do
	stateheaders[i] = CreateFrame("Frame", "StateHeader", nil, "SecureStateHeaderTemplate");
end

---------------------------------------------------------------------------------------------
-- Setup/Profile Functions
---------------------------------------------------------------------------------------------
function QH_OnLoad()

	-- Setup Slash Commands
	QH_SetupCommands();
	
	-- Set Events
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
end

function QH_SetupCommands()

	-- Toggles QH on and off
	SlashCmdList["QH"] = QH_ShowOptions;
	SLASH_QH1 = "/QH";
	SLASH_QH2 = "/QuickHeal";
end

function QH_Setup()
	
	-- Initialize Profile Variables
	QH_InitializeProfile();

	-- Store Default TargetFrame Position
	QHtfpoint,QHtfrelativeTo,QHtfrelativePoint,QHtfxofs,QHtfyofs = getglobal("TargetFrame"):GetPoint();
		
	-- Setup UI
	QH_SetupUI();
end

function QH_SetupUI()

	-- Update Status Bars
	QH_UpdateStatusBars("player");
	QH_UpdateStatusBars("party1");
	QH_UpdateStatusBars("party2");
	QH_UpdateStatusBars("party3");
	QH_UpdateStatusBars("party4");

	-- Update Buttons
	QH_SetupUnitButton("player");
	QH_SetupUnitButton("party1");
	QH_SetupUnitButton("party2");
	QH_SetupUnitButton("party3");
	QH_SetupUnitButton("party4");

	-- Adjust Target Frame To Accomodate For Player Status Bar
	getglobal("TargetFrame"):ClearAllPoints();
	if(QHrunning) then

		getglobal("TargetFrame"):SetPoint(QHtfpoint, QHtfrelativeTo, QHtfrelativePoint, QHtfxofs + 100, QHtfyofs);
	else
		getglobal("TargetFrame"):SetPoint(QHtfpoint, QHtfrelativeTo, QHtfrelativePoint, QHtfxofs, QHtfyofs);
	end
end

function QH_InitializeProfile()

	if ( UnitName("player") ) then
		PlayerName = UnitName("player").."|"..QH_Trim(GetCVar("realmName"));
		QH_LoadSettings();
	end

	QH_Report();
end

function QH_LoadSettings()

	if ( QHProfile[PlayerName] == nil ) then
		QHProfile[PlayerName] = {};
		QHProfile[PlayerName]["QHrunning"] = 'true';
		QHProfile[PlayerName]["QHplayerleft"] = nil;
		QHProfile[PlayerName]["QHplayermiddle"] = nil;
		QHProfile[PlayerName]["QHplayerright"] = nil;
		QHProfile[PlayerName]["QHpartyleft"] = nil;
		QHProfile[PlayerName]["QHpartymiddle"] = nil;
		QHProfile[PlayerName]["QHpartyright"] = nil;
	end

	QHrunning = QHProfile[PlayerName]["QHrunning"];
	QHplayerleft = QHProfile[PlayerName]["QHplayerleft"];
	QHplayermiddle = QHProfile[PlayerName]["QHplayermiddle"];
	QHplayerright = QHProfile[PlayerName]["QHplayerright"];
	QHpartyleft = QHProfile[PlayerName]["QHpartyleft"];
	QHpartymiddle = QHProfile[PlayerName]["QHpartymiddle"];
	QHpartyright = QHProfile[PlayerName]["QHpartyright"];
end

function QH_ResetSettings()

	QHProfile[PlayerName] = nil;
	QH_LoadSettings();
	QH_UpdateOptions();
	QH_SetupUI();
end

function QH_SaveVar(varname, value)

	if ( PlayerName ) then
		if ( QHProfile[PlayerName] ) then
			QHProfile[PlayerName][varname] = value;
		end
	end
end

---------------------------------------------------------------------------------------------
-- Event Handlers
---------------------------------------------------------------------------------------------
function QH_OnEvent()

        if (event == "VARIABLES_LOADED") then

		QH_Setup();

	elseif(event == "PLAYER_LEAVE_COMBAT" or event == "PLAYER_REGEN_ENABLED") then

		QH_SetupUI();
	end
end

---------------------------------------------------------------------------------------------
-- Options GUI Functions
---------------------------------------------------------------------------------------------
function QH_ShowOptions()

	if (QHOptionsFrame:IsVisible()) then
		HideUIPanel(QHOptionsFrame);
	else
		QH_UpdateOptions();
		ShowUIPanel(QHOptionsFrame);
	end
end

function QH_UpdateOptions()

	-- Setup CheckBoxes
	QHBOXONOFF:SetChecked(QHrunning);
	
	-- Setup Dropdown Menus
	UIDropDownMenu_Initialize(QHplayerleftdropdown, QH_InitializeDropdown);
	UIDropDownMenu_SetWidth(150, QHplayerleftdropdown);
	UIDropDownMenu_SetText("", QHplayerleftdropdown);
	UIDropDownMenu_SetSelectedValue(QHplayerleftdropdown, QHplayerleft);

	UIDropDownMenu_Initialize(QHplayermiddledropdown, QH_InitializeDropdown);
	UIDropDownMenu_SetWidth(150, QHplayermiddledropdown);
	UIDropDownMenu_SetText("", QHplayermiddledropdown);
	UIDropDownMenu_SetSelectedValue(QHplayermiddledropdown, QHplayermiddle);

	UIDropDownMenu_Initialize(QHplayerrightdropdown, QH_InitializeDropdown);
	UIDropDownMenu_SetWidth(150, QHplayerrightdropdown);
	UIDropDownMenu_SetText("", QHplayerrightdropdown);
	UIDropDownMenu_SetSelectedValue(QHplayerrightdropdown, QHplayerright);

	UIDropDownMenu_Initialize(QHpartyleftdropdown, QH_InitializeDropdown);
	UIDropDownMenu_SetWidth(150, QHpartyleftdropdown);
	UIDropDownMenu_SetText("", QHpartyleftdropdown);
	UIDropDownMenu_SetSelectedValue(QHpartyleftdropdown, QHpartyleft);

	UIDropDownMenu_Initialize(QHpartymiddledropdown, QH_InitializeDropdown);
	UIDropDownMenu_SetWidth(150, QHpartymiddledropdown);
	UIDropDownMenu_SetText("", QHpartymiddledropdown);
	UIDropDownMenu_SetSelectedValue(QHpartymiddledropdown, QHpartymiddle);

	UIDropDownMenu_Initialize(QHpartyrightdropdown, QH_InitializeDropdown);
	UIDropDownMenu_SetWidth(150, QHpartyrightdropdown);
	UIDropDownMenu_SetText("", QHpartyrightdropdown);
	UIDropDownMenu_SetSelectedValue(QHpartyrightdropdown, QHpartyright);

end

function QH_InitializeDropdown()

	local spells = "";
	info = UIDropDownMenu_CreateInfo();

	-- Populate list with available spells in spellbook
	local i=1; while true do

   		local spellname, spellrank = GetSpellName(i, BOOKTYPE_SPELL);

		-- Exit loop if nil
   		if not spellname then
		do break end
   		end

		-- Add Spell to Menu
		if((IsSpellInRange(spellname, "player") or IsSpellInRange(spellname, "party")) and not IsPassiveSpell(i, BOOKTYPE_SPELL) and strfind(spells, spellname) == nil) then

			spells = spells .. spellname .. ";";
		end
		
   		i=i+1;
	end

	-- Sort Spell List
	slist = {strsplit(";", spells)};
	sort(slist);

	-- Add Spells to Dropdown
	for i=1,#(slist) do
		
		info = {};	
		info.text = slist[i];--spellname;
		info.value = slist[i];--spellname;
		info.owner = getglobal(UIDROPDOWNMENU_INIT_MENU);
		info.func = QHDropdown_OnClick;
			
		UIDropDownMenu_AddButton(info);
	end
end

function QHDropdown_OnClick()

	-- Set Dropdown Value	
	UIDropDownMenu_SetSelectedValue(this.owner, this.value);
	
	-- Save Settings
	local var = gsub(this.owner:GetName(), "(dropdown)", "");
	setglobal(var, this.value);
	QH_SaveVar(var, this.value);

	-- Update Unit Buttons
	QH_SetupUI();
end

function QHOptionsCheckButton_OnLoad()

	-- Setup Check Boxes
	if(this == QHBOXONOFF) then
		getglobal(this:GetName().."Text"):SetText(QH_LOCALE_RUNNING);
	end

end

function QHOptionsCheckButton_OnClick()

	-- Check which box was clicked
	if(this == QHBOXONOFF) then

		QHrunning = this:GetChecked();
		QH_SaveVar("QHrunning", QHrunning);

		-- Update Unit Buttons
		QH_SetupUI();
	end
end

---------------------------------------------------------------------------------------------
-- Spell Functions
---------------------------------------------------------------------------------------------
function QH_SetupUnitButton(unit)
	
	local button = this;
	if(unit ~= nil) then
		button = getglobal("QHButton" .. unit);
	end
	
	if(QHrunning) then

		button:Enable();
		local id = button:GetID();

		--button:SetScript("OnAttributeChanged", QH_ButtonUpdate);

		--Setup States
		--button:SetAttribute("newstate", "*:1");
		--button:SetAttribute("statebutton", "1:heal1;2:heal2");
		--stateheaders[id]:SetAttribute("addchild", button);
		--stateheaders[id]:SetAttribute("state", "1");
		
		button:RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');

		local localizedClass, englishClass = UnitClass("player");

		if(unit == "player") then

			button:SetAttribute("unit", "player");

			-- LEFT CLICK
			button:SetAttribute("*helpbutton1", "playerleft");
			button:SetAttribute("*type-playerleft", "spell");
			button:SetAttribute("spell-playerleft", QHplayerleft);

			-- RIGHT CLICK
			button:SetAttribute("*helpbutton2", "playerright");
			button:SetAttribute("*type-playerright", "spell");
			button:SetAttribute("spell-playerright", QHplayerright);

			-- MIDDLE CLICK
			button:SetAttribute("*helpbutton3", "playermiddle");
			button:SetAttribute("*type-playermiddle", "spell");
			button:SetAttribute("spell-playermiddle", QHplayermiddle);
		
		else

			button:SetAttribute("unit", "party"..id-1);

			-- LEFT CLICK
			button:SetAttribute("*helpbutton1", "partyleft");
			button:SetAttribute("*type-partyleft", "spell");
			button:SetAttribute("spell-partyleft", QHpartyleft);

			-- RIGHT CLICK
			button:SetAttribute("*helpbutton2", "partyright");
			button:SetAttribute("*type-partyright", "spell");
			button:SetAttribute("spell-partyright", QHpartyright);

			-- MIDDLE CLICK
			button:SetAttribute("*helpbutton3", "partymiddle");
			button:SetAttribute("*type-partymiddle", "spell");
			button:SetAttribute("spell-partymiddle", QHpartymiddle);
		end
	else

		button:Disable();
	end
end

---------------------------------------------------------------------------------------------
-- UI Functions
---------------------------------------------------------------------------------------------
function QH_UpdateStatusBars(unit)

	local bar = getglobal("QHBar"..unit);
	
	if(QHrunning and bar ~= nil) then

		bar:Show();

		if(unit == "player" or unit == "party1" or unit == "party2" or unit == "party3" or unit == "party4") then

			local health = UnitHealth(unit);
			
			local button = getglobal("QHButton"..unit);
			local healthmax = UnitHealthMax(unit);
	
			bar:SetMinMaxValues(0, healthmax);

			if(UnitIsDeadOrGhost(unit)) then

				bar:SetValue(healthmax);
				bar:SetStatusBarColor(1, 0, 0);
				stateheaders[button:GetID()]:SetAttribute("state", "2"); --Only works out of combat

			elseif(not UnitIsConnected(unit)) then

				bar:SetValue(0);
				bar:SetStatusBarColor(0, 0, 0);
				stateheaders[button:GetID()]:SetAttribute("state", "1"); --Only works out of combat

			elseif(health and healthmax) then

				bar:SetValue(healthmax - health);

				if((health / healthmax) < 0.66) then
					bar:SetStatusBarColor(1, 1, 0);
				else
					bar:SetStatusBarColor(0, 1, 0);
				end

				stateheaders[button:GetID()]:SetAttribute("state", "1"); --Only works out of combat
			end
		end

	elseif(bar ~=nil) then

		bar:Hide();
	end
end

---------------------------------------------------------------------------------------------
-- Utility Functions
---------------------------------------------------------------------------------------------
--TODO: Add color
function QH_Print(msg, red, green, blue)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

function QH_Trim(s)
	return (string.gsub(s, "^%s*(.-)%s*$", "%1"));
end

---------------------------------------------------------------------------------------------
-- Slash Command Functions
---------------------------------------------------------------------------------------------
function QH_Report()
	if(QHrunning) then
		QH_Print("QuickHeal is |cFF00FF00ON".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	else
		QH_Print("QuickHeal is |cFFFF0000OFF".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	end
end

function QH_Help()

	QH_Print("|cFFFFFF00"..QH_LOCALE_HELP);
end

function QH_ToggleRun()

	if(QHrunning) then
		QHrunning = nil;
		QH_Print("QuickHeal is now |cFFFF0000OFF".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	else
		QHrunning = 'true';
		QH_Print("QuickHeal is now |cFF00FF00ON".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	end
end